@@ -1,8 +1,7 @@
 """CloudWatch Embedded Metric Format utility
 """
-from aws_lambda_powertools.helper.models import MetricUnit
-
-from .exceptions import MetricUnitError, MetricValueError, SchemaValidationError, UniqueNamespaceError
+from .base import MetricUnit
+from .exceptions import MetricUnitError, MetricValueError, SchemaValidationError
 from .metric import single_metric
 from .metrics import Metrics
 
@@ -13,5 +12,4 @@ __all__ = [
     "MetricUnitError",
     "SchemaValidationError",
     "MetricValueError",
-    "UniqueNamespaceError",
 ]
